/* -*-c-*-
 *
 * The SHA256 hash function
 *
 * (c) 2020 Mark Wooding
 */

/*----- Licensing notice --------------------------------------------------*
 *
 * This file is part of Runlisp, a tool for invoking Common Lisp scripts.
 *
 * Runlisp is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 3 of the License, or (at your
 * option) any later version.
 *
 * Runlisp is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Runlisp.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef SHA256_H
#define SHA256_H

#ifdef __cplusplus
  extern "C" {
#endif

/*----- Header files ------------------------------------------------------*/

#include <limits.h>

/*----- Data types --------------------------------------------------------*/

/* Select a suitable type for 32-bit words. */
#if UINT_MAX >= 0xffffffff
  typedef unsigned u32;
#else
  typedef unsigned long u32;
#endif

#define SHA256_BLKSZ 64			/* input block size in bytes */
#define SHA256_HASHSZ 32		/* output hash size in bytes */

struct sha256_state {
  unsigned n;				/* number of live bytes in buffer */
  size_t nblk;				/* number of blocks hashed so far */
  u32 a[8];				/* hash state */
  unsigned char buf[SHA256_BLKSZ];	/* input buffer */
};

/*----- Functions provided ------------------------------------------------*/

extern void sha256_init(struct sha256_state */**/);
	/* Initialize the hash state S. */

extern void sha256_hash(struct sha256_state */*s*/,
			const void */*m*/, size_t /*sz*/);
	/* Append SZ bytes of data starting at M to the hash state S. */

extern void sha256_done(struct sha256_state */*s*/, unsigned char */*h*/);
	/* Write the final hash of state S to buffer H. */

/*----- That's all, folks -------------------------------------------------*/

#ifdef __cplusplus
  }
#endif

#endif
